# A makefile include for generic Java projects.
# To use:
#    - PROG must be the program's name (output $(PROG).jar)
#    - SRC must be a list of java files.
#    - MAIN_CLASS must be defined.
#    - The following variables can otherwise be overridden.
#
# Targets defined:
#   all      - (default) builds the target jar file.
#   run      - Runs the program.
#   javadoc  - Builds javadoc documentation.
#   clean    - Removes temporary files, class files.
#   cleanall - Removes clean's files, as well as javadoc and build targets.


# Semicolon in Windows, colon in UNIX.
# Semicolon must be escaped for bash.
#CLASSPATH_SEP ?= \;
CLASSPATH_SEP ?= :

#JAVA_DIR ?= c:/prog/java
JAVA_DIR ?= /usr

CLASS_DIR       ?= class
MAIN_CLASS      ?= $(PROG)
JAR_FILE	?= $(PROG).jar
MANIFEST_FILE	?= $(CLASS_DIR)/manifest
CLS = $(SRC:%.java=$(CLASS_DIR)/%.class)

JAVAC    ?= $(JAVA_DIR)/bin/javac
JAVADOC  ?= $(JAVA_DIR)/bin/javadoc
JAVA     ?= $(JAVA_DIR)/bin/java
JAR      ?= $(JAVA_DIR)/bin/jar
JUNIT_JAR ?= ./junit.jar
RM       ?= rm -f
#JFLAGS   ?= -O
JFLAGS   ?=
RUNARGS	 ?=

### The rest of the file probably doesn't need to be touched.

# Extend CLASSPATH to include the current directory.
ifdef CLASSPATH
CLASSPATH := .$(CLASSPATH_SEP)$(CLASSPATH)
else
CLASSPATH = .
endif

default: all

all:	$(JAR_FILE)

$(JAR_FILE): $(SRC) javafiles $(MANIFEST_FILE)
	if [ -n "$(CLASS_DIR)" -a ! -d $(CLASS_DIR) ]; then mkdir $(CLASS_DIR); fi
	$(JAVAC) -classpath $(CLASSPATH) $(JFLAGS) -d $(CLASS_DIR) @javafiles
	if [ -d $(PROG)/images ]; then cp -r $(PROG)/images $(CLASS_DIR)/$(PROG); fi
	cd $(CLASS_DIR); \
	find . -name "*.class" > classfiles; \
	find . -name "*.gif" >> classfiles; \
	$(JAR) cmf ../$(MANIFEST_FILE) ../$(JAR_FILE) @classfiles

$(MANIFEST_FILE): Makefile
	if [ -n "$(CLASS_DIR)" -a ! -d $(CLASS_DIR) ]; then mkdir $(CLASS_DIR); fi
	if [ ! -f $(MANIFEST_FILE) ]; then echo "Main-Class: $(MAIN_CLASS)" > $(MANIFEST_FILE); fi

run: $(JAR_FILE)
	$(JAVA) -classpath $(JAR_FILE)$(CLASSPATH_SEP)$(CLASSPATH) $(MAIN_CLASS) $(RUNARGS)

javafiles: Makefile
	echo $(SRC) > javafiles

javadoc: $(SRC) javafiles
	if [ ! -d doc ]; then mkdir doc; fi
	if [ ! -d doc/javadoc ]; then mkdir doc/javadoc; fi
	$(JAVADOC) -d doc/javadoc -classpath $(CLASSPATH) @javafiles

clean:
	$(RM) `find . -name "*~"`
	$(RM) -r $(CLASS_DIR)
	$(RM) javafiles classfiles testOutput

cleanall: clean
	$(RM) -r doc/javadoc
	$(RM) $(JAR_FILE)
	if [ -n "$(CLASS_DIR)" -a -d $(CLASS_DIR) ]; then rmdir $(CLASS_DIR); fi
