package rangesearch.api;

import jdsl.core.api.*;
import jdsl.core.ref.*;

/** An interface describing a two level range tree for 2D range over 2D
    points. */

public interface RangeTree2D {
    /** Initializes the range tree to the given set of elements, with the given
     * comparators.
     * @param comp1 this defines how items will be compared in one dimension 
     *              of the search and defines the primary level of the range
     *              tree
     * @param comp2 this defines how items will be compared in the second
     *              dimension of the search and defines the secondary level
     *              of the range tree
     */
    public void initialize(ObjectIterator oi,
                           Comparator comp1,
                           Comparator comp2);

    /** Queries the elements used to initialize the range tree and returns an
	iterator over the objects within the bounds given.
     * @param start1 the element defining the lower bound for the primary structure.
     * @param end1 the element defining the upper bound for the primary structure.
     * @param start2 the element defining the lower bound for the secondary structure.
     * @param end2 the element defining the upper bound for the secondary structure.
     * @return an iterator over the elements in the given ranges.
     * @throws RangeSearchException if either start variable is above its
     *          respective end variable or if the input objects cannot be 
     *          compared with the data 
     */
    public ObjectIterator query(Object start1, Object end1,
                                Object start2, Object end2)
        throws RangeSearchException;
}

