package rangesearch.test;

import rangesearch.api.*;
import rangesearch.ref.*;
import rangesearch.*;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;
import junit.framework.TestCase;

public class RangeTree2DImplTest extends RangeSearch2DUtil 
{
    public RangeTree2DImplTest(String name) {
        super(name);
    }

    public void testSquare() {
        PublicRangeSearch2D prs = new PublicRangeSearch2D();
        int coords[] = {
            10, 10,
            10, 20,
            20, 20, 
            20, 10,
            15, 15
        };
        prs.initialize(buildPointSequence(coords));
        // XXX - JDSL quick sort is unstable?!  Expected:
        //  "[[(10, 10) (10, 20)] [(15, 15) [(20, 20) (20, 10)]]]",
        assertEquals(
            "[[(10, 20) (10, 10)] [(15, 15) [(20, 10) (20, 20)]]]",
            prs.getTreeRepr());

        BinaryTree tree = prs.getTree();
        Sequence s = (Sequence) tree.root().element();
        Point2DIteratorAdapter pi = new Point2DIteratorAdapter(s.elements());
        assertEquals("[ (10, 10) (10, 20) (15, 15) (20, 10) (20, 20) ]",
                     JDSLUtil.pointListToString(pi));
    }

    class PublicRangeTree2D extends RangeTree2DImpl {
	public String getTreeRepr() {
	    return getNodeRepr(m_tree.root());
	}
	public BinaryTree getTree() {
	    return m_tree;
	}
	protected String getNodeRepr(Position node) {
	    String s;
	    if(m_tree.isExternal(node)) {
		Sequence seq = (Sequence) node.element();
		Point2D p = (Point2D) seq.atRank(0).element();
		s = JDSLUtil.pointToString(p);
	    } else {
		s = "[" + getNodeRepr(m_tree.leftChild (node)) +
		    " " + getNodeRepr(m_tree.rightChild(node)) +
		    "]";
	    }
	    return s;
	}
    }
    
    class PublicRangeSearch2D {
	protected PublicRangeTree2D m_rangeTree;
	protected GeomTester2D m_tester;
	
	public String getTreeRepr() {
	    return m_rangeTree.getTreeRepr();
	}
	
	public BinaryTree getTree() {
	    return m_rangeTree.getTree();
	}
	
	public PublicRangeSearch2D() {
	    m_rangeTree = null;
	    m_tester = new GeomTester2DImpl();
	}
	
	/** Initializes the range searcher with the given set of points.
	 */
	public void initialize(Point2DIterator pi) {
	    Comparator lrComparator = new LeftRightComparator(m_tester);
	    Comparator abComparator = new AboveBelowComparator(m_tester);
	    m_rangeTree = new PublicRangeTree2D();
	    m_rangeTree.initialize(pi, lrComparator, abComparator);
	}
	
	/** Queries the current set of points using the query rectangle r.
	 */
	public Point2DIterator query(Rectangle2D r) {
	    Assertion.check(m_rangeTree != null, "query called before initialize.");
	    Assertion.check(r != null, "null query range");
	    
	    Point2D p1 = r.topLeftPoint();
	    Point2D p2 = r.bottomRightPoint();
	    ObjectIterator oi = m_rangeTree.query(p1, p2, p1, p2);
	    return (Point2DIterator) oi;
	}
    }
}
